# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2010 Raoul Snyman                                        #
# Portions copyright (c) 2008-2010 Tim Bentley, Jonathan Corwin, Michael      #
# Gorven, Scott Guerrieri, Christian Richter, Maikel Stuivenberg, Martin      #
# Thompson, Jon Tibble, Carsten Tinggaard                                     #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import os

from PyQt4 import QtCore, QtGui
import logging

from songusagedetaildialog import Ui_SongUsageDetailDialog

log = logging.getLogger(__name__)

class SongUsageDetailForm(QtGui.QDialog, Ui_SongUsageDetailDialog):
    log.info(u'SongUsage Detail Form loaded')
    """
    Class documentation goes here.
    """
    def __init__(self, parent=None):
        """
        Constructor
        """
        QtGui.QDialog.__init__(self, None)
        self.parent = parent
        self.setupUi(self)

    def initialise(self):
        year = QtCore.QDate().currentDate().year()
        if QtCore.QDate().currentDate().month() < 9:
            year -= 1
        toDate = QtCore.QDate(year, 8, 31)
        fromDate = QtCore.QDate(year - 1, 9, 1)
        self.FromDate.setSelectedDate(fromDate)
        self.ToDate.setSelectedDate(toDate)
        self.FileLineEdit.setText(self.parent.config.get_last_dir(1))

    def defineOutputLocation(self):
        path = QtGui.QFileDialog.getExistingDirectory(self,
            self.trUtf8('Output File Location'),
            self.parent.config.get_last_dir(1) )
        path = unicode(path)
        if path != u'':
            self.parent.config.set_last_dir(path, 1)
            self.FileLineEdit.setText(path)

    def accept(self):
        log.debug(u'Detailed report generated')
        filename = u'usage_detail_%s_%s.txt' % \
            (self.FromDate.selectedDate().toString(u'ddMMyyyy'),
             self.ToDate.selectedDate().toString(u'ddMMyyyy'))
        usage = self.parent.songusagemanager.get_all_songusage(\
                                    self.FromDate.selectedDate(), \
                                    self.ToDate.selectedDate())
        outname = os.path.join(unicode(self.FileLineEdit.text()), filename)
        file = None
        try:
            file = open(outname, u'w')
            for instance in usage:
                record = u'\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"\n' % \
                    (instance.usagedate,instance.usagetime, instance.title,
                    instance.copyright, instance.ccl_number , instance.authors)
                file.write(record)
        except:
            log.exception(u'Failed to write out song usage records')
        finally:
            if file:
                file.close()

